use bevy::render::primitives::Plane;
use bevy::{math::bool, prelude::shape::Quad, prelude::*};
use rand::prelude::*;

enum EyePos {
    Left,
    Right,
}

#[derive(Default, Component)]
pub struct ScreenInfo {
    pub name: String, // main (to identify the screen)
    pub width: f32,       // 3.08    (full width in m)
    pub height: f32,      // 2.33    (full height in m)

    pub center: Vec3, // 0.0 -1.15 -3.08 (mid point of screen in global coordinated - tracking!)
    // normal: Vec3, // 0.0 0.0 1.0      (orientation of front side)
    // up: Vec3,     // 0.0 1.0 0.0      (vertical axis)
    // horizontal: Vec3, // right vector computed as orthonormal

}

impl ScreenInfo {
    pub fn new(name: &str) -> Self {
        Self {
            name: name.to_string(),
            width: 3.08,
            height: 2.33,
            center: Vec3 {
                x: 0.0,
                y: 0.0,
                z: 0.0,
            },
            // normal: Vec3::Z,
            // up: Vec3::Y,
            // horizontal: Vec3::ZERO,
        }
    }

    pub fn corner_points(&self) -> (Vec3,Vec3,Vec3,Vec3) {

        let lower_left = Vec3::new(self.center.x - self.width * 0.5,self.center.y - self.height * 0.5,0.0);
        let lower_right = Vec3::new(self.center.x + self.width * 0.5,self.center.y - self.height * 0.5,0.0);
        let upper_left = Vec3::new(self.center.x - self.width * 0.5,self.center.y + self.height * 0.5,0.0);
        let upper_right = Vec3::new(self.center.x + self.width * 0.5,self.center.y + self.height * 0.5,0.0);

        (lower_left,lower_right,upper_left,upper_right)
    }

    pub fn build_debug_geometry(
        &self,
        mut commands: Commands,
        mut meshes: ResMut<Assets<Mesh>>,
        mut materials: ResMut<Assets<StandardMaterial>>,
    ) {
        // plane
        commands.spawn(PbrBundle {
            mesh: meshes.add(Mesh::from(Quad {
                size: Vec2 {
                    x: self.width,
                    y: self.height,
                },
                flip: false,
            })),
            material: materials.add(Color::rgb(0.3, 0.5, 0.3).into()),
            ..default()
        });
    }

    // pub fn update(&mut self) -> &mut Self {
    //     // compute right as up ^ forward
    //     self.horizontal = self.up.cross(self.normal);

    //     // points of screen
    //     self.top = self.center + self.up * (self.height * 0.5);
    //     self.bottom = self.center - self.up * (self.height * 0.5);
    //     self.left = self.center - self.horizontal * (self.height * 0.5);
    //     self.right = self.center + self.horizontal * (self.height * 0.5);

    //     self
    // }



}


